---------------------------------------------------------------------------------
--
-- Prat - A framework for World of Warcraft chat mods
--
-- Copyright (C) 2006-2007  Prat Development Team
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc., 
-- 51 Franklin Street, Fifth Floor, 
-- Boston, MA  02110-1301, USA.
--
--
-------------------------------------------------------------------------------



-- Get Utility Libraries
local util, DBG, CLR = GetPratUtils()
local Chat_Type = {
	"SAY",
	"PARTY",
	"RAID",
	"RAID_LEADER",
	"RAID_WARNING",
	"GUILD",
	"OFFICER",
	"YELL",
	"WHISPER",
	"BATTLEGROUND",
	"BATTLEGROUND_LEADER",
}
-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("PratChatHead")

if PRAT_MODULE == nil then 
	return 
end

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
	["ChatHead"] = true,
	["ChatHead options."] = true,
	["Using ChatHead in channel options."] = true,
	["%s channel chat head settings."] = true,
	["Use"] = true,
	["Toggle using chat head in %s channel."] = true,
	["ChatHead text"] = true,
	["Set ChatHead text used in %s channel."] = true,
	["Common setting"] = true,
	["Use common setting"] = true,
	["Use common chathead"] = true,
	["Set common chathead"] = true,
	["Set common chathead."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["ChatHead"] = "말머리",
	["ChatHead options."] = "말머리 모듈의 설정을 변경합니다.",
	["Using ChatHead in channel options."] = "채널에 사용할 말머리에 관한 설정을 변경합니다.",
	["%s channel chat head settings."] = "%s 채널에 사용할 말머리 기능을 설정합니다.",
	["Use"] = "사용",
	["Toggle using chat head in %s channel."] = "%s 채널에 말머리 사용 여부를 선택합니다",
	["ChatHead text"] = "말머리 문구",
	["Set ChatHead text used in %s channel."] = "%s 채널에서 사용할 말머리를 설정합니다",
	["Common setting"] = "공통 설정",
	["Use common setting"] = "공통 말머리 사용에 관한 설정을 변경합니다",
	["Use common chathead"] = "공통 말머리를 사용합니다",
	["Set common chathead"] = "공통 말머리 설정",
	["Set common chathead."] = "공통 말머리로 사용할 문구를 지정합니다.",
} end)

-- get prat module categories
local cat = Prat.Categories

-- create prat module
Prat_ChatHead = Prat:NewModule(PRAT_MODULE)
Prat_ChatHead.revision = tonumber(string.sub("$Revision: 35398 $", 12, -3))

-- define key module values
Prat_ChatHead.moduleName = L["ChatHead"]
Prat_ChatHead.moduleDesc = L["ChatHead options."]
Prat_ChatHead.consoleName = "ChatHead"
Prat_ChatHead.guiName = L["ChatHead"]
Prat_ChatHead.Categories = { cat.TEXT, cat.BEHAVIOR }

-- Set this to true to use Prat Event Support
Prat_ChatHead.supportsPratEvents = true

-- define the default db values
Prat_ChatHead.defaultDB = {
	on = false,
	custom = { },
	common = {  },
	chathead = {
		["SAY"] = {  },
		["PARTY"] = {  },
		["RAID"] = {  },
		["GUILD"] = {  },
		["OFFICER"] = {  },
		["YELL"] = {  },
		["WHISPER"] = {  },
		["RAID_WARNING"] = {  },
		["RAID_LEADER"] = {  },
		["BATTLEGROUND"] = {  },
		["BATTLEGROUND_LEADER"] = {  },
	}
}

-- create a moduleOptions stub (for setting self.moduleOptions)
Prat_ChatHead.moduleOptions = {}


-- add module options not covered by templates
function Prat_ChatHead:GetModuleOptions()
	self.moduleOptions = {
		name = L["ChatHead"],
		desc = L["Using ChatHead in channel options."],
		type = "group",
		args = {
			alluse = {
				name = L["Common setting"],
				desc = L["Use common setting"],
				type = "group",
				order = 2,
				args = {
					use = {
						name = L["Use"],
						desc = L["Use common chathead"],
						type = "toggle",
						order = 1,
						get = function() return Prat_ChatHead.db.profile.common.use end,
						set = function() Prat_ChatHead.db.profile.common.use = not Prat_ChatHead.db.profile.common.use end,
					},
					set = {
						name = L["Set common chathead"],
						desc = L["Set common chathead."],
						type = "text",
						usage = "<string>",
						order = 2,
						get = function() return Prat_ChatHead.db.profile.common.ch end,
						set = function(v) Prat_ChatHead.db.profile.common.ch = v end,
					},
				},
			},
		}
	}
	self:BuildChannelOptions()
	return self.moduleOptions
end

function CLR:ChannelColor(text, type) return self:Colorize(Prat_ChatHead:GetChatCLR(type), text) end

function Prat_ChatHead:GetChatCLR(name)
	local type = strsub(name, 10);
	self:Debug(type)
	local info = ChatTypeInfo[type];
	if not info then
		return CLR.COLOR_NONE
	end
	return CLR:GetHexColor(info)
end

function Prat_ChatHead:CreateTypeOption(args, name, order)
	local text = TEXT("CHAT_MSG_"..name)

	text = CLR:ChannelColor(TEXT(getglobal("CHAT_MSG_"..name)), text)

	args[name] = {
		name = text,
		desc = string.format(L["%s channel chat head settings."], text),
		type = "group",
		order = order + 101,
		args = {
			use = {
				name = L["Use"],
				desc = string.format(L["Toggle using chat head in %s channel."], text),
				order = 1,
				type = "toggle",
				get = function() return self.db.profile.chathead[name].use end,
				set = function(v) self.db.profile.chathead[name].use = v end
			},
			set = {
				name = L["ChatHead text"],
				desc = string.format(L["Set ChatHead text used in %s channel."], text),
				order = 2,
				disabled = function() return not self.db.profile.chathead[name].use end,
				type = "text",
				usage = "<string>",
				get = function() return self.db.profile.chathead[name].ch end,
				set = function(v) self.db.profile.chathead[name].ch = v end
			},
		}
	}
end

function Prat_ChatHead:CreateChannelOption(args, chanNum)
	local _, chanName = GetChannelName(chanNum)
	if chanName ~= nil then
		_, _,  chanName = string.find(chanName, "([^%s]+)")
		local text = TEXT("CHAT_MSG_CHANNEL"..chanNum)
		text = CLR:ChannelColor(chanName, text)
		if not self.db.profile.chathead[chanName] then
			self.db.profile.chathead[chanName] = {  }
		end
		args[chanName] = {
			name = text,
			desc = string.format(L["%s channel chat head settings."], text),
			type = "group",
			order = 200 + chanNum,
			args = {
				use = {
					name = L["Use"],
					desc = string.format(L["Toggle using chat head in %s channel."], text),
					order = 1,
					type = "toggle",
					get = function() return self.db.profile.chathead[chanName].use end,
					set = function(v) self.db.profile.chathead[chanName].use = v end
				},
				set = {
					name = L["ChatHead text"],
					desc = string.format(L["Set ChatHead text used in %s channel."], text),
					order = 2,
					disabled = function() return not self.db.profile.chathead[chanName].use end,
					type = "text",
					usage = "<string>",
					get = function() return self.db.profile.chathead[chanName].ch end,
					set = function(v) self.db.profile.chathead[chanName].ch = v end
				},
			}
		}
	end
end

function Prat_ChatHead:BuildChannelOptions()
	for k, v in ipairs(Chat_Type) do
		self:CreateTypeOption(self.moduleOptions.args, v, k)
	end
	for i = 1, 10 do
		self:CreateChannelOption(self.moduleOptions.args, i)
	end
end

--[[------------------------------------------------
Module Event Functions
------------------------------------------------]]--

-- things to do when the module is enabled
function Prat_ChatHead:OnModuleEnable()
	self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE", "BuildChannelOptions")
	-- what da hook gon be?
	self:Hook("SendChatMessage", true)
	self:BuildChannelOptions()
end

-- things to do when the module is disabled
function Prat_ChatHead:OnModuleDisable()
	-- unhook functions
	self:UnhookAll()
end

--[[------------------------------------------------
Core Functions
------------------------------------------------]]--

function Prat_ChatHead:SendChatMessage(msg, chatType, language, channel)
	if msg and not string.find(msg, "^/(.+)") then
		if self.db.profile.common.use then
			msg = self.db.profile.common.ch..msg
		else
			if chatType == "CHANNEL" then
				local _, chanName = GetChannelName(channel)
				_, _, chanName = string.find(chanName, "([^%s]+)")
				if self.db.profile.chathead[chanName].use and self.db.profile.chathead[chanName].ch ~= "" and self.db.profile.chathead[chanName].ch ~= nil then
					msg = self.db.profile.chathead[chanName].ch..msg
				end
			elseif self.db.profile.chathead[chatType] and self.db.profile.chathead[chatType].use and self.db.profile.chathead[chatType].ch ~= "" and self.db.profile.chathead[chatType].ch ~= nil then
				msg = self.db.profile.chathead[chatType].ch..msg
			end
		end
	end
	self.hooks.SendChatMessage(msg, chatType, language, channel)
end